/**
 * js logic specific to opening and closing the panels from the site header elements
 */

import domMethods from "~/dom-methods";
import "~/polyfills/nodelist-for-each-polyfill";

const panelSelector = ".o-panel";
const panelCatSelector = ".o-panel-categories";
const panelSearchSelector = ".o-panel-search";
const panelBgHitSelector = ".o-panel-bg-area";
const menuButtonSelector = ".c-control--menu-button";
const searchButtonSelector = ".c-control--search-button";
const closeControlSelector = ".c-control--close";
const panelOpenClass = "o-panel--open";
const panelCatsOpenClass = "o-panel-categories--open";
const panelSearchOpenClass = "o-panel-search--open";

let menuButton;
let panel;
let panelCategories;
let panelSearch;
let panelBgHit;
let closeControl;
let searchButton;
let panelObject = {};

/**
 * @method init
 * @description init method, adds event listener for scroll and resize
 * @return void
 */
panelObject.init = () => {
  panel = document.querySelector(panelSelector);
  panelCategories = document.querySelector(panelCatSelector);
  panelSearch = document.querySelector(panelSearchSelector);
  menuButton = document.querySelectorAll(menuButtonSelector);
  searchButton = document.querySelectorAll(searchButtonSelector);
  closeControl = document.querySelectorAll(closeControlSelector);

  menuButton.forEach((el) => {
    el.addEventListener("click", panelObject.onCategoriesClicked);
  });

  closeControl.forEach((el) => {
    el.addEventListener("click", panelObject.panelClose);
  });

  searchButton.forEach((el) => {
    el.addEventListener("click", panelObject.onSearchClicked);
  });
};

/**
 * @method onCategoriesClicked
 * @description on menu clicked
 * @return void
 */
panelObject.onCategoriesClicked = () => {
  const isMenuOpen = panel.classList.contains(panelOpenClass);
  const isCatOpen = panelCategories.classList.contains(panelCatsOpenClass);

  isMenuOpen && isCatOpen ? panelObject.panelClose() : panelObject.categoriesOpen();
};

/**
 * @method categoriesOpen
 * @description adds open classes
 * @return void
 */
panelObject.categoriesOpen = () => {
  panel.classList.add(panelOpenClass);
  panelCategories.classList.add(panelCatsOpenClass);
  panelSearch.classList.remove(panelSearchOpenClass);

  // if first element return false, then focus the second element
  !domMethods.focusElementInPanel(".c-search__input") && domMethods.focusElementInPanel(".c-accordion__item__link");

  domMethods.ariaToggle(".c-control--menu-button .c-control__button", "close navigation", true);
  panelObject.navigationFade();
  panelObject.addBgPanel();
};

/**
 * @method onSearchClicked
 * @description adds open class
 * @return void
 */
panelObject.onSearchClicked = () => {
  const isMenuOpen = panel.classList.contains(panelOpenClass);
  const isSearchOpen = panelSearch.classList.contains(panelSearchOpenClass);

  isMenuOpen && isSearchOpen ? panelObject.panelClose() : panelObject.searchOpen();
};

/**
 * @method searchOpen
 * @description adds open class
 * @return void
 */
panelObject.searchOpen = () => {
  panel.classList.add(panelOpenClass);
  panelSearch.classList.add(panelSearchOpenClass);
  panelCategories.classList.remove(panelCatsOpenClass);

  // if first element return false, then focus the second element
  !domMethods.focusElementInPanel(".c-search__input") && domMethods.focusElementInPanel(".c-accordion__item__link");

  domMethods.ariaToggle(".c-control--search-button .c-control__button", "close search", true);
  panelObject.navigationFade();
  panelObject.addBgPanel();
};

/**
 * @method panelClose
 * @description removes open classes
 * @return void
 */
panelObject.panelClose = () => {
  panel.classList.remove(panelOpenClass);
  panelCategories.classList.remove(panelCatsOpenClass);
  panelSearch.classList.remove(panelSearchOpenClass);

  domMethods.ariaToggle(".c-control--menu-button .c-control__button", "open navigation", false);
  domMethods.ariaToggle(".c-control--search-button .c-control__button", "open search", false);
  panelObject.navigationUnFade();
  panelObject.removeBgPanel();

  let focusElements = document.querySelectorAll(".c-control--menu-button .c-control__button");

  Array.from(focusElements).some((m) => {
    let isVisible = m.offsetHeight !== 0;
    if (isVisible) {
      m.focus();
    }
    return isVisible;
  });
};

/**
 * @method addBgPanel
 * @description adds the hidden background element
 * @return void
 */
panelObject.addBgPanel = () => {
  const doesBgExist = document.querySelector(".o-panel-bg-area") !== null;

  if (!doesBgExist) {
    const divPanelBG = document.createElement("div");
    divPanelBG.classList.add("o-panel-bg-area");
    document.body.appendChild(divPanelBG);
    panelBgHit = document.querySelector(panelBgHitSelector);
    panelBgHit.addEventListener("click", panelObject.panelClose);
  } else {
    document.querySelector(panelBgHitSelector).classList.remove("o-panel-bg-area--closed");
  }
};

/**
 * @method removeBgPanel
 * @description removes the hidden background element
 * @return void
 */
panelObject.removeBgPanel = () => {
  document.querySelector(panelBgHitSelector).classList.add("o-panel-bg-area--closed");
};

/**
 * @method navigationFade
 * @description adds fade class to specific elements
 * @return void
 */
panelObject.navigationFade = () => {
  const navBar = document.querySelector(".c-header-nav__bar");
  const h2 = document.querySelector("h2");
  const ignoreClasses = ["c-control--menu-button", "c-control--search-button"];
  const headerElement = document.querySelector(".c-header-nav") ||
    document.querySelector(".c-header-title--article") ||
    document.querySelector(".c-header-title--section") ||
    document.querySelector(".c-header-title--variant");
  const allControlButtons = headerElement.querySelectorAll(".c-control");
  const fadeClass = "u-faded";

  allControlButtons.forEach((c) => {
    let notAnIgnoredClass = ignoreClasses.every((ignoreString) => {
      return !c.classList.contains(ignoreString);
    });

    notAnIgnoredClass && c.classList.add(fadeClass);
  });

  navBar && navBar.classList.add(fadeClass);
  h2 && h2.classList.add(fadeClass);
};

/**
 * @method navigationUnFade
 * @description removes fade class to specific elements
 * @return void
 */
panelObject.navigationUnFade = () => {
  const navBar = document.querySelector(".c-header-nav__bar");
  const h2 = document.querySelector("h2");
  const headerElement = document.querySelector(".c-header-nav") ||
    document.querySelector(".c-header-title--article") ||
    document.querySelector(".c-header-title--section") ||
    document.querySelector(".c-header-title--variant");
  const allControlButtons = headerElement.querySelectorAll(".c-control");
  const fadeClass = "u-faded";

  allControlButtons.forEach((c) => {
    c.classList.remove(fadeClass);
  });

  navBar && navBar.classList.remove(fadeClass);
  h2 && h2.classList.remove(fadeClass);
};

export default panelObject;
